@@ -40,12 +40,12 @@ class Query(object):
         In general a query object should be created via ``Session.query``, 
         not directly.
     '''
-    def __init__(self, type, db):
+    def __init__(self, type, session):
         '''**Parameters**:
                 * type: A subclass of class:`mongoalchemy.document.Document`
-                * db: The ``pymongo`` database which this query is associated with.
+                * db: The :class:`~mongoalchemy.session.Session` which this query is associated with.
         '''
-        self.db = db
+        self.session = session
         self.type = type
         self.query = {}
         self.sort = []
@@ -58,25 +58,16 @@ class Query(object):
         return self.__get_query_result()
     
     def __get_query_result(self):
-        collection = self.db[self.type.get_collection_name()]
-        for index in self.type.get_indexes():
-            index.ensure(collection)
-        
-        kwargs = dict()
-        if self._fields:
-            kwargs['fields'] = [str(f) for f in self._fields]
-        
-        cursor = collection.find(self.query, **kwargs)
-        
-        if self.sort:
-            cursor.sort(self.sort)
-        if self.hints:
-            cursor.hint(self.hints)
-        if self._limit != None:
-            cursor.limit(self._limit)
-        if self._skip != None:
-            cursor.skip(self._skip)
-        return QueryResult(cursor, self.type, fields=self._fields)
+        return self.session.execute_query(self)
+    
+    def get_fields(self):
+        return self._fields
+    
+    def get_limit(self):
+        return self._limit
+    
+    def get_skip(self):
+        return self._skip
     
     def limit(self, limit):
         '''Sets the limit on the number of documents returned
@@ -99,7 +90,7 @@ class Query(object):
             updates to the cloned object or the original object will not 
             affect each other
         '''
-        qclone = Query(self.type, self.db)
+        qclone = Query(self.type, self.session)
         qclone.query = deepcopy(self.query)
         qclone.sort = deepcopy(self.sort)
         qclone._fields = deepcopy(self._fields)
@@ -185,7 +176,7 @@ class Query(object):
             .. seealso:: :class:`~mongoalchemy.query_expression.QueryExpression` class
         '''
         for qe in query_expressions:
-            self.__apply(qe)
+            self._apply(qe)
         return self
     
     def count(self, with_limit_and_skip=False):
@@ -212,7 +203,7 @@ class Query(object):
             self._fields.add(f)
         return self
     
-    def __apply(self, qe):
+    def _apply(self, qe):
         ''' Apply a query expression, updating the query object '''
         for k, v in qe.obj.iteritems():
             if k not in self.query:
@@ -250,18 +241,18 @@ class Query(object):
         self.sort.append((name, direction))
         return self
     
-    def not_(self, *query_expressions):
-        ''' Add a $not expression to the query, negating the query expressions 
-            given.  
-            
-            **Examples**: ``query.not_(SomeDocClass.f.age == 18)`` becomes ``{'$not' : { 'age' : 18 }}``
-            
-            **Parameters**:
-            * query_expressions: Instances of :class:`mongoalchemy.query_expression.QueryExpression`
-            '''
-        for qe in query_expressions:
-            self.filter(qe.not_())
-        return self
+    # def not_(self, *query_expressions):
+    #     ''' Add a $not expression to the query, negating the query expressions 
+    #         given.  
+    #         
+    #         **Examples**: ``query.not_(SomeDocClass.f.age == 18)`` becomes ``{'$not' : { 'age' : 18 }}``
+    #         
+    #         **Parameters**:
+    #         * query_expressions: Instances of :class:`mongoalchemy.query_expression.QueryExpression`
+    #         '''
+    #     for qe in query_expressions:
+    #         self.filter(qe.not_())
+    #     return self
     
     def or_(self, first_qe, *qes):
         ''' Add a $not expression to the query, negating the query expressions 
@@ -352,3 +343,57 @@ class QueryResult(object):
         return self
 
 
+class RemoveQuery(object):
+    def __init__(self, type, session):
+        ''' Execute a remove query to remove the matched objects from the database
+            **Parameters**:
+                * type: A subclass of class:`mongoalchemy.document.Document`
+                * db: The :class:`~mongoalchemy.session.Session` which this query is associated with.
+        '''
+        self.session = session
+        self.type = type
+        self.safe = None
+        self.get_last_args = {}
+        self.query = {}
+        self.__query_obj = Query(type, session)
+    
+    def set_safe(self, is_safe, **kwargs):
+        ''' Set this remove to be safe.  It will call getLastError after the 
+            remove to make sure it was successful.  ``**kwargs`` are parameters to 
+            MongoDB's getLastError command (as in pymongo's remove).
+        '''
+        self.safe = is_safe
+        self.get_last_args.update(**kwargs)
+        return self
+    
+    def execute(self):
+        ''' Run the remove command on the session.  Return the result of 
+            ``getLastError`` if ``safe`` is ``True``'''
+        return self.session.execute_remove(self)
+    
+    def filter(self, *query_expressions):
+        ''' Filter the remove expression with ``*query_expressions``, as in
+            the ``Query`` filter method.'''
+        self.__query_obj.filter(*query_expressions)
+        self.query = self.__query_obj.query
+        return self
+    
+    # def not_(self, *query_expressions):
+    #     self.__query_obj.not_(*query_expressions)
+    #     self.query = self.__query_obj.query
+    #     return self
+        
+    def or_(self, first_qe, *qes):
+        ''' Works the same as the query expression method ``or_``
+        '''
+        self.__query_obj.or_(first_qe, *qes)
+        self.query = self.__query_obj.query
+        return self
+    
+    def in_(self, qfield, *values):
+        ''' Works the same as the query expression method ``in_``
+        '''
+
+        self.__query_obj.in_(qfield, *values)
+        self.query = self.__query_obj.query
+        return self
\ No newline at end of file
